#!/usr/bin/env python
from __future__ import print_function
import sys
import json
import hashlib


# hetzner-auction.py - display auction json
# ed <irc.rizon.net>, MIT-licensed, 2018-09-20, v1.0
# https://ocv.me/dev/?hetzner-auction.py
#
# usage:
#   cd ~/Documents/hetzner-cache/
#   wget https://www.hetzner.com/a_hz_serverboerse/live_data.json
#   ~/dev/hetzner-auction.py "$(find -maxdepth 1 -type f -iname 'live_data.json*' -printf '%T@ %p\n' | sort -n | tail -n 1 | sed -r 's/[^ ]* //')" | less -R 


class Reg(object):
	def __init__(self, json_text):
		data = json.loads(json_text)
		#print(json.dumps(data['server'], indent=4))
		#sys.exit(0)
		self.boxes = data['server']
		self.cmin = 20
		self.cmax = 231
	
	def getcolor(self, text, ofs):
		rv = int(hashlib.md5(text).hexdigest()[ofs:ofs+8], 16)
		return self.cmin + divmod(rv, self.cmax - self.cmin)[1]
	
	def tint(self, text, sz):
		return '\033[0;38;5;{0}m{1}\033[0m'.format(
			self.getcolor(text, sz), text)
	
	def draw(self):
		for b in self.boxes:
			b_red = b['next_reduce_hr'].strip()
			b_dc  = b['datacenter'][0].strip()
			b_cpu = b['cpu'].strip()
			b_pm  = b['cpu_benchmark']
			b_ram = b['ram']
			b_nhd = b['hdd_count']
			b_shd = b['hdd_size']
			b_trf = b['traffic'].strip()
			b_bw  = b['bandwith'] / 1000
			b_sp  = b['specials']
			
			if b_nhd < 2 \
			or b_shd < 2000 \
			or b_pm < 1500 \
			or 'Ent. HDD' not in b_sp:
				continue
			
			ccpu = self.getcolor(b_cpu, 6)
			
			cpm = 1
			if b_pm > 8000:
				cpm = 5
			if b_pm > 9000:
				cpm = 3
			if b_pm > 9700:
				cpm = '2;1'
			if b_pm > 9900:
				cpm = '7;1'
				
			cram = 1
			if b_ram >= 16:
				cram = 3
			if b_ram >= 32:
				cram = '2;1'
			if b_ram >= 48:
				cram = '7;1'
			
			chdd = 1
			if b_shd >= 2000:
				chdd = 3
			if b_shd >= 3000:
				chdd = '2;1'
			if b_shd >= 4000:
				chdd = '7;1'
			
			cdc = '0;1'
			if b_dc in [
				'HEL1-DC2', 'FSN1-DC13', 'FSN1-DC12', 'NBG1-DC3']:
				cdc = '7;1'
			
			cred = '5'
			hred = int(b_red.split('h')[0], 10)
			if hred <= 24:
				cred = '4;1'
			if hred < 10:
				cred = '2'
			if hred == 0:
				cred = '3;1'
			if b_red == '00h 00m':
				cred = '0;1'
			
			specials = '] ['.join(self.tint(x, 8) for x in b_sp)
			
			print('{eur:6.2f}  \033[0;3{cdc}m{dc:<10} \033[0;38;5;{ccpu}m{cpu:<22}  \033[0;3{cpm}m{pm:>5} pm  \033[0;3{cram}m{ram:>6}  \033[0;3{chdd}m{hdn:>2}x {hds:<5}  \033[0;36m{trf:<6}  {bw} gbit  \033[0;3{cred}m{red:>8}  \033[0m[{sp}]'.format(
				eur = float(b['price']),
				red = b_red,
				dc  = b_dc,
				cpu = b_cpu,
				pm  = b_pm,
				ram = b['ram_hr'].strip(),
				hdn = b_nhd,
				hds = str(b_shd/1024) + ' TB',
				trf = b_trf,
				bw  = b_bw,
				sp  = specials,
				cdc  = cdc,
				ccpu = ccpu,
				cpm  = cpm,
				cram = cram,
				chdd = chdd,
				cred = cred
			))


with open(sys.argv[1], 'rb') as f:
	json_text = f.read().decode('utf-8')

r = Reg(json_text)
r.draw()
